/*
 *	sotel - SSL and OTP telnet for Windows CE
 *  Copyright(C) 1998 NEC Corporation
 *	All rights reserved.
 *	Author: Mizukoshi, Yasuhiro <mizukosi@ccs.mt.nec.co.jp>
 *	E-Mail: pochi@ccs.mt.nec.co.jp
 */

#include	"sotel.h"
#include	"resource.h"

int   hash_func;
int   seq;
char  seed[SEED_MAX_LEN];
char  pass[ONETIME_KEY_MAX_LEN];
int   (* KeyCrunchFunc)(char *, char *, char *);
void  (* IterateFunc)  (char *);
TCHAR FuncName[16];

char  RecvBuf[MAX_OTPBUF_SIZE + 1];
int   RecvBuf_pos = 0;

struct {
	TCHAR *key;
	int   func;
} keys[] = {
	{ TEXT("s/key"),     MD4 },
	{ TEXT("331 s/key"), MD4 },
	{ TEXT("S/Key MD4"), MD4 },
	{ TEXT("S/Key MD5"), MD5 },
	{ TEXT("otp-md4"),   MD4 },
	{ TEXT("otp-md5"),   MD5 },
	{ TEXT("otp-sha1"),  SHA1 },
	{ TEXT("331 otp-md4"),  MD4 },
	{ TEXT("331 otp-md5"),  MD5 },
	{ TEXT("331 otp-sha1"), SHA1 },
	{ TEXT("331 OTP response otp-md4"),  MD4 },
	{ TEXT("331 OTP response otp-md5"),  MD5 },
	{ TEXT("331 OTP response otp-sha1"), SHA1 },
	{ TEXT("331 Response to otp-md4"),   MD4 },
	{ TEXT("331 Response to otp-md5"),   MD5 },
	{ TEXT("331 Response to otp-sha1"),  SHA1 },
	{ NULL, 0 },
};

int SetFuncName()
{
	switch(hash_func){
	case MD4:
		KeyCrunchFunc = (void *)_otp_MD4_keycrunch;
		IterateFunc   = (void *)_otp_MD4_iterate;
		wcscpy(FuncName, FUNCNAME_MD4);
		break;
	case MD5:
		KeyCrunchFunc = (void *)_otp_MD5_keycrunch;
		IterateFunc   = (void *)_otp_MD5_iterate;
		wcscpy(FuncName, FUNCNAME_MD5);
		break;
	case SHA1:
		KeyCrunchFunc = (void *)_otp_SHA1_keycrunch;
		IterateFunc   = (void *)_otp_SHA1_iterate;
		wcscpy(FuncName, FUNCNAME_SHA1);
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

int chkResp()
{
	int   i, j, klen, slen;
	TCHAR *p, *ps;
	TCHAR tRecvBuf[MAX_OTPBUF_SIZE + 1];
	TCHAR seed_tmp[SEED_MAX_LEN];

	mbstowcs(tRecvBuf, RecvBuf, sizeof(tRecvBuf));

	for (i = 0; keys[i].key != NULL; i++){
		klen = wcslen(keys[i].key);
		if ( !wcsncmp(tRecvBuf, keys[i].key, klen) ){
			hash_func = keys[i].func;
			/* for S/Key MD5 */
			if ( bSKeyMD5 && ((0 <= i) && (i <= 1)) )
				hash_func = MD5;
			SetFuncName();
			p = tRecvBuf + klen + 1;
			/* V[PXԍ̎擾 */
			seq = _wtoi(p);
			/* V[h̎擾 */
			j = 0;
			ps = NULL;
			while (p && *p){
				if (!iswalnum(*p)){
					j++;
					if (j == 2) break;
				} else if (j == 1 && ps == NULL){
					ps = p;
				}
				p++;
			}
			if (j == 1 && ps) j++;
			if (j == 2){
				slen = p - ps;
				wcsncpy(seed_tmp, ps, slen);
				seed_tmp[slen] = '\0';
				wcstombs(seed, seed_tmp, sizeof(seed));
			} else {
				return FALSE;
			}
			return TRUE;
		}
	}
	return FALSE;
}

void OtpCheck(SOCKET s, char *buf, int len)
{
	int		i;
	int		res;

	for (i = 0; i < len; i++){
		if (RecvBuf_pos >= MAX_OTPBUF_SIZE){
			RecvBuf[0]  = 0;
			RecvBuf_pos = 0;
		}
		if (buf[i] == '\n'){
			RecvBuf[RecvBuf_pos] = 0;
			RecvBuf_pos = 0;

			if ( chkResp() ){
				res = DialogBox(hInst, MAKEINTRESOURCE(IDD_PASSPHRASE),
								NULL, PassphraseDlgProc);
				if (res){
					char sbuf[ONETIME_KEY_MAX_LEN + 8];

					/* one-time password̑M */
					strcpy(sbuf, pass);
					strcat(sbuf, CRLF);
					SendTelnet(s, sbuf, strlen(sbuf));
				}
			}
		} else {
			RecvBuf[RecvBuf_pos++] = buf[i];
		}
	}
	return;
}

BOOL CALLBACK PassphraseDlgProc(HWND hWnd, UINT uMessage,
				WPARAM wParam, LPARAM lParam)
{
    switch(uMessage) {
    case WM_INITDIALOG:
	{
		TCHAR Title[SEED_MAX_LEN + 10];
		TCHAR strtmp[SEED_MAX_LEN + 1];

		SetForegroundWindow(hWnd);
		mbstowcs(strtmp, seed, sizeof(strtmp));
		wsprintf(Title, TEXT("%d %s"), seq, strtmp);
		SetWindowText(hWnd, Title);		
		SetWindowText(GetDlgItem(hWnd, IDC_FUNC), FuncName);
	}
		break;
	case WM_COMMAND:
		switch(wParam){
		case IDOK:
		{
			TCHAR passphrase_tmp[PASS_PHRASE_MAX_LEN];
			char  passphrase[PASS_PHRASE_MAX_LEN];
			char  key[8];
			int   i;

			/* one-time passworďvZ */
			GetDlgItemText(hWnd, IDC_PASS, passphrase_tmp, PASS_PHRASE_MAX_LEN);
			wcstombs(passphrase, passphrase_tmp, sizeof(passphrase));
			if (KeyCrunchFunc(key, seed, passphrase) == -1){
				EndDialog(hWnd, FALSE);
				break;
			}
			for (i = 0; i < seq; i++) IterateFunc(key);
			OTP_btoe(pass, key);
			EndDialog(hWnd, TRUE);
		}
			break;
		case IDCANCEL:
			EndDialog(hWnd, FALSE);
			break;
		default:
			return FALSE;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}																	   	
